@@ -19,7 +19,7 @@ except ImportError:
     # Python 3.
     import urllib.parse as urlparse
 
-ovstools_script = '''#!/bin/bash
+ovstools_script = '''
 #!/bin/bash
 #Transfer the ip address from $bridge to $physicial interface.
 # INPUT:
@@ -50,18 +50,26 @@ function ovsif_takeover()
     ip addr flush dev $ifname
     echo "ip addr flush dev $ifname"
 
-    if [[ $ifname == $port ]]
-    then
-        # Add the physicial interface to the bridge $br
-        ovs-vsctl add-port $br $ifname
-        echo "ovs-vsctl add-port $br $ifname"
-    fi
+    # Add the physicial interface to the bridge $br
+    ovs-vsctl --may-exist add-port $br $port
+    echo "ovs-vsctl --may-exist add-port $br $port"
+
+    #if [[ $ifname == $port ]]
+    #then
+    #    # Add the physicial interface to the bridge $br
+    #    ovs-vsctl add-port $br $ifname
+    #    echo "ovs-vsctl add-port $br $ifname"
+    #fi
 
     #ovs-vsctl set bridge $br other_config:hwaddr=$mac
     ip link set $br up
     echo "ip link set $br up"
-    ip address add $ip4 dev $br
-    echo "ip address add $ip4 dev $br"
+    if [ -n "$ip4" ]; then
+        ip address add $ip4 dev $br
+        echo "ip address add $ip4 dev $br"
+    else
+        echo "ip4 is empty, ignore..."
+    fi
 
     # transfer the route from $ifname to $br
     for line in ${route[@]};
@@ -244,7 +252,8 @@ class OvsHost(linux.LinuxDevice):
         '''
         ip = kwargs.pop("ip", []) #get the ip configuration of the bridge
         # Process the ports and bonds specially:
-        ports = kwargs.pop("port", []) # get the ports list
+        ports = list(args)
+        ports.extend(kwargs.pop("port", [])) # get the ports list
         bonds = kwargs.pop("bond", {}) # get the ports list
         # Create the kwargs
         command = 'ovs-vsctl --may-exist add-br %s' %(name)
@@ -261,9 +270,6 @@ class OvsHost(linux.LinuxDevice):
         if self.cmd("ovs-vsctl get-fail-mode %s" %(name)).strip() == "secure":
             self.ofctl_add_flow(name, "priority=0,actions=drop")
 
-        # Add self-port if any, it's port = xxx, or *args,
-        ports = ports if (isinstance(ports, list)) else [ports]
-        ports = ports if (not args) else ports.extend(args)
         for p in ports:
             p["mac"] = self.add_port(name, **p)
         if bonds:
@@ -271,8 +277,8 @@ class OvsHost(linux.LinuxDevice):
 
         if ip:
             # Configure the ip address for the address for route
-            for p in ports:
-                self.cmd("ip addr flush dev %s" %(p))
+            #for p in ports:
+            #    self.cmd("ip addr flush dev %s" %(p["name"]))
             self.cmd('ip link set %s up' %(name))
             self.cmd('ip address add %s dev %s' %(ip, name))
             #self.cmd('ovs-appctl ovs/route/add %s %s' %(ip, name))
@@ -299,11 +305,11 @@ class OvsHost(linux.LinuxDevice):
         Add a port to the brige and do some configuration if necessary.
         return the mac address of the port if has.
         '''
-        if (kwargs.get("type", "normal") == "normal") and \
-                (name in self.nics):
-            #If it's a normal interface, take over it
-            self.takeover_interface(bridge_name, name, name)
-            return None
+        #if (kwargs.get("type", "normal") == "normal") and \
+        #        (name in self.nics):
+        #    #If it's a normal interface, take over it
+        #    self.takeover_interface(name, bridge_name, name)
+        #    return None
 
         command = 'ovs-vsctl --may-exist add-port %s %s' %(bridge_name, name)
 
@@ -321,6 +327,9 @@ class OvsHost(linux.LinuxDevice):
         macaddr = self.ovs_get_interface(name, "mac_in_use").strip('"')
         #kwargs["mac"] = macaddr
         self.macs.append(macaddr)
+        if (kwargs.get("type", "normal") == "normal"):
+            # For normal port, flush it and then set the ip if there is any.
+            self.cmd("ip addr flush dev %s" %(name))
         if kwargs.get("ip", None) :
             # Configure the ip address for the address for route
             self.cmd('ip link set %s up' %(name))
@@ -336,9 +345,9 @@ class OvsHost(linux.LinuxDevice):
                     )
         return macaddr
 
-    def takeover_interface (self, bridge_name, ifname, portname = None):
+    def takeover_interface (self, ifname, bridge, port = None):
         '''
-        takeover the ip from @ifname to @bridge_name(@portname)
+        takeover the ip from @ifname to @bridge(@port)
         '''
         filename = '.netdevice_ovstools.sh'
         if self.cmd("[ -f ~/%s ] || echo no" %(filename)).strip() == "no":
@@ -347,11 +356,11 @@ class OvsHost(linux.LinuxDevice):
             self.put_file(filename, "~/")
 
         self.cmd("sh ~/%s takeover %s %s %s" %(filename,
-            ifname, bridge_name, portname and portname or ifname))
+            ifname, bridge, port and port or ifname))
 
-    def no_takeover_interface (self, bridge_name, ifname, portname = None):
+    def no_takeover_interface (self, ifname, bridge, port = None):
         '''
-        restore the ip from @bridge_name(@portname) to @ifname
+        restore the ip from @bridge(@port) to @ifname
         '''
         filename = ".netdevice_ovstools.sh"
         if self.cmd("[ -f ~/%s ] || echo no" %(filename)).strip() == "no":
@@ -360,7 +369,7 @@ class OvsHost(linux.LinuxDevice):
             self.put_file(filename, "~/")
 
         self.cmd("sh ~/%s restore %s %s %s" %(filename,
-            ifname, bridge_name, portname and portname or ifname))
+            ifname, bridge, port and port or ifname))
 
     def add_device (self, bridge_name, *args, **kwargs):
         '''
@@ -632,7 +641,7 @@ class OvsHost(linux.LinuxDevice):
                 #self.log("debug, bridge: %s, port: %s" %(b, p))
                 if (p in self.nics):
                     # Restore the "phisical" interface
-                    self.no_takeover_interface(b, p, p)
+                    self.no_takeover_interface(p, b, p)
                 elif ("=" in external_ids):
                     # Parse external_ids: {container_id=con13, container_iface=eth1}
                     i = {}
@@ -741,20 +750,45 @@ class OvsHost(linux.LinuxDevice):
                 self.log("FAIL: %s ping %s, %s -> %s!" %(src["name"], dst["name"],
                     srcip.ip, dstip.ip), bg_color = "red")
 
-    def vm_init2 (self, vm, script = None, gateway = None, remote = None):
-        # configure route and execute the script if given
-        u = urlparse.urlparse(vm["url"])
-        c = "ip route add %s via %s dev %s" \
-                %(ipaddress.ip_interface(remote["ip"]).network,
-                    ipaddress.ip_interface(gateway["ip"]).ip, vm["port"][1]["name"])
-        if script:
-            self.cmd("sshpass -p %s scp %s %s@%s:~/" %(u.password, script,
-                u.username, u.hostname))
-            c += "; sh ~/%s" %(script)
-        self.cmd('sshpass -p %s ssh %s@%s "%s"' %(u.password, u.username, u.hostname, c))
+    def _util_dictionary_parse (self, params):
+        '''
+        Add some flows to ofproto
+        '''
+        my_dic = {}
+        if "=" not in params[0]:
+            my_dic["type"] = params.pop(0)
+        for attribute in params:
+            if ("=" in attribute):
+                k,v = attribute.split("=", 1)
+                my_dic[k.strip()] = v.strip()
+            else:
+                my_dic[attribute] = ""
+        return my_dic
 
     def vm_init (self, *args, gateways = [], script = None):
-        # Given some devivces and gates, configure route automatically
+        '''
+        Given some devivces and gates, caculate the route and configure route
+        automatically.
+
+        @args: The list of vm, the vm is a dictionary, it at least has the
+        following part:
+
+            vm11 = {'name': 'vm11',
+            'url': 'ssh://root:sangfor@10.103.166.36',
+            'port': [{'name': 'ens3', 'ip': '10.103.166.36'},
+                {'name': 'ens4', 'ip': '10.208.1.11'}]}
+
+        @gateways: The list of gateway. The gateway is a dictionary, it at
+        least has the following part:
+
+            gw1 = {"name": "gw1", "ip": "10.208.1.1/24"}
+
+        @script: the script to run when login the rm.
+
+        An example:
+
+            host.vm_init(*devices, gateways = [gw1, gw2]) # configure route
+        '''
 
         devices = copy.deepcopy(args)
         #subnets = (ipaddress.ip_interface(gw["ip"]).network for gw in gateways)
@@ -788,26 +822,27 @@ class OvsHost(linux.LinuxDevice):
                 u.username, u.hostname, c))
 
     def vm_uninit (self, *args):
-        # configure route and execute the script if given
+        '''
+        Power off the virtual machine from this host.
+        从这台主机上将给定的虚拟机关机。
+
+        @args: The list of the vm to power on. Itis a dictionary and at least
+        has the following part:
+        @args: 需要关机的虚拟机列表。虚拟机用字典定义，应该至少具有下面的成员
+        ：
+
+            vm11 = {'url': 'ssh://root:sangfor@10.103.166.36'}
+
+        An example:
+        一个调用的例子：
+
+            h1.vm_uninit(*devices)
+        '''
         for vm in args:
             u = urlparse.urlparse(vm["url"])
             self.cmd('sshpass -p %s ssh %s@%s shutdown now' %(u.password,
                 u.username, u.hostname))
 
-    def _util_dictionary_parse (self, params):
-        '''
-        Add some flows to ofproto
-        '''
-        my_dic = {}
-        if "=" not in params[0]:
-            my_dic["type"] = params.pop(0)
-        for attribute in params:
-            if ("=" in attribute):
-                k,v = attribute.split("=", 1)
-                my_dic[k.strip()] = v.strip()
-            else:
-                my_dic[attribute] = ""
-        return my_dic
 
 if __name__ == '__main__':
     '''
